import unittest

import sys
sys.path.append('../')

from urllib3.connectionpool import HTTPConnectionPool, get_host, connection_from_url, HostChangedError

class TestConnectionPool(unittest.TestCase):
    def test_get_host(self):
        url_host_map = {
            'http://google.com/mail': ('http', 'google.com', None),
            'http://google.com/mail/': ('http', 'google.com', None),
            'google.com/mail': ('http', 'google.com', None),
            'http://google.com/': ('http', 'google.com', None),
            'http://google.com': ('http', 'google.com', None),
            'http://www.google.com': ('http', 'www.google.com', None),
            'http://mail.google.com': ('http', 'mail.google.com', None),
            'http://google.com:8000/mail/': ('http', 'google.com', 8000),
            'http://google.com:8000': ('http', 'google.com', 8000),
            'https://google.com': ('https', 'google.com', None),
            'https://google.com:8000': ('https', 'google.com', 8000),
        }
        for url, expected_host in url_host_map.iteritems():
            returned_host = get_host(url)
            self.assertEquals(returned_host, expected_host)

    def test_same_host(self):
        same_host = [
            ('http://google.com/', '/'),
            ('http://google.com/', 'http://google.com/'),
            ('http://google.com/', 'http://google.com'),
            ('http://google.com/', 'http://google.com/abra/cadabra'),
            ('http://google.com:42/', 'http://google.com:42/abracadabra'),
        ]

        for a,b in same_host:
            c = connection_from_url(a)
            self.assertTrue(c.is_same_host(b), "%s =? %s" % (a, b))

        not_same_host = [
            ('http://yahoo.com/', 'http://google.com/'),
            ('http://google.com:42', 'https://google.com/abracadabra'),
            ('http://google.com', 'https://google.net/'),
        ]

        for a,b in not_same_host:
            c = connection_from_url(a)
            self.assertFalse(c.is_same_host(b), "%s =? %s" % (a,b))




import httplib

import mimetools, mimetypes

ENCODE_TEMPLATE= """--%(boundary)s
Content-Disposition: form-data; name="%(name)s"

%(value)s
""".replace('\n','\r\n')

ENCODE_TEMPLATE_FILE = """--%(boundary)s
Content-Disposition: form-data; name="%(name)s"; filename="%(filename)s"
Content-Type: %(contenttype)s

%(value)s
""".replace('\n','\r\n')

# TODO: is replace('\n', '\r\n') going to cause problems on other platforms?
# Are we better off building a list of strings and doing '\r\n'.join(body)?

def get_content_type(filename):
    return mimetypes.guess_type(filename)[0] or 'application/octet-stream'

def encode_multipart_formdata(fields):
    """
    Given a dictionary field parameters, returns the HTTP request body and the
    content_type (which includes the boundary string), to be used with an
    httplib-like call.

    Normal key/value items are treated as regular parameters, but key/tuple
    items are treated as files, where a value tuple is a (filename, data) tuple.

    For example:

    fields = {
        'foo': 'bar',
        'foofile': ('foofile.txt', 'contents of foofile'),
    }

    body, content_type = encode_multipart_formdata(fields)
    """

    BOUNDARY = mimetools.choose_boundary()

    body = ""

    # NOTE: Every non-binary possibly-unicode variable must be casted to str()
    # because if a unicode value pollutes the `body` string, then all of body
    # will become unicode. Appending a binary file string to a unicode string
    # will cast the binary data to unicode, which will raise an encoding
    # exception. Long story short, we want to stick to plain strings.
    # This is not ideal, but if anyone has a better method, I'd love to hear it.

    for key, value in fields.iteritems():
        if isinstance(value, tuple):
            filename, value = value
            body += ENCODE_TEMPLATE_FILE % {
                        'boundary': BOUNDARY,
                        'name': str(key),
                        'value': str(value),
                        'filename': str(filename),
                        'contenttype': str(get_content_type(filename))
                    }
        else:
            body += ENCODE_TEMPLATE % {
                        'boundary': BOUNDARY,
                        'name': str(key),
                        'value': str(value)
                    }

    body += '--%s--\n\r' % BOUNDARY
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY

    return body, content_type



import unittest

import sys
sys.path.append('../')

from urllib3 import HTTPConnectionPool, TimeoutError, MaxRetryError

HOST="localhost"
PORT=8081

import logging
log = logging.getLogger('urllib3.connectionpool')
log.setLevel(logging.NOTSET)
log.addHandler(logging.StreamHandler(sys.stdout))

class TestConnectionPool(unittest.TestCase):
    def __init__(self, *args, **kw):
        # Test for dummy server...
        self.http_pool = HTTPConnectionPool(HOST, PORT)
        try:
            r = self.http_pool.get_url('/', retries=1)
            if r.data != "Dummy server!":
                raise Exception("Got unexpected response: %s" % r.data)
        except Exception, e:
            raise Exception("Dummy server not running, make sure HOST and PORT correspond to the dummy server: %s" % e.message)

        return super(TestConnectionPool, self).__init__(*args, **kw)

    def test_get_url(self):
        r = self.http_pool.get_url('/specific_method', fields={'method': 'GET'})
        self.assertEquals(r.status, 200, r.data)

    def test_post_url(self):
        r = self.http_pool.post_url('/specific_method', fields={'method': 'POST'})
        self.assertEquals(r.status, 200, r.data)

    def test_urlopen_put(self):
        r = self.http_pool.urlopen('PUT', '/specific_method?method=PUT')

    def test_wrong_specific_method(self):
        # To make sure the dummy server is actually returning failed responses
        r = self.http_pool.get_url('/specific_method', fields={'method': 'POST'})
        self.assertEquals(r.status, 400, r.data)

        r = self.http_pool.post_url('/specific_method', fields={'method': 'GET'})
        self.assertEquals(r.status, 400, r.data)

    def test_upload(self):
        data = "I'm in ur multipart form-data, hazing a cheezburgr"
        fields = {'upload_param': 'filefield',
                'upload_filename': 'lolcat.txt',
                'upload_size': len(data),
                'filefield': ('lolcat.txt', data)}

        r = self.http_pool.post_url('/upload', fields=fields)
        self.assertEquals(r.status, 200, r.data)

    def test_timeout(self):
        pool = HTTPConnectionPool(HOST, PORT, timeout=0.1)
        try:
            r = pool.get_url('/sleep', fields={'seconds': 0.2})
            self.fail("Failed to raise TimeoutError exception")
        except TimeoutError, e:
            pass

    def test_redirect(self):
        r = self.http_pool.get_url('/redirect', fields={'target': '/'}, redirect=False)
        self.assertEquals(r.status, 303)

        r = self.http_pool.get_url('/redirect', fields={'target': '/'})
        self.assertEquals(r.status, 200)
        self.assertEquals(r.data, 'Dummy server!')

    def test_maxretry(self):
        try:
            r = self.http_pool.get_url('/redirect', fields={'target': '/'}, retries=0)
            self.fail("Failed to raise MaxRetryError exception")
        except MaxRetryError, e:
            pass




from connectionpool import HTTPConnectionPool, HTTPSConnectionPool, get_host, connection_from_url
from filepost import encode_multipart_formdata

# Possible exceptions
from connectionpool import HTTPError, MaxRetryError, TimeoutError



#!/usr/bin/env python
# Really simple rudimentary benchmark to compare ConnectionPool versus standard
# urllib to demonstrate the usefulness of connection re-using.

# URLs to download. Doesn't matter as long as they're from the same host, so we
# can take advantage of connection re-using.
TO_DOWNLOAD = [
    'http://code.google.com/apis/apps/',
    'http://code.google.com/apis/base/',
    'http://code.google.com/apis/blogger/',
    'http://code.google.com/apis/calendar/',
    'http://code.google.com/apis/codesearch/',
    'http://code.google.com/apis/contact/',
    'http://code.google.com/apis/books/',
    'http://code.google.com/apis/documents/',
    'http://code.google.com/apis/finance/',
    'http://code.google.com/apis/health/',
    'http://code.google.com/apis/notebook/',
    'http://code.google.com/apis/picasaweb/',
    'http://code.google.com/apis/spreadsheets/',
    'http://code.google.com/apis/webmastertools/',
    'http://code.google.com/apis/youtube/',
]

import sys
sys.path.append('../')

from urllib3 import HTTPConnectionPool
import urllib
import time

def urllib_get(url_list):
    assert url_list
    for url in url_list:
        now = time.time()
        r = urllib.urlopen(url)
        elapsed = time.time() - now
        print "Got in %0.3f: %s" % (elapsed, url)

def pool_get(url_list):
    assert url_list
    pool = HTTPConnectionPool.from_url(url_list[0])
    for url in url_list:
        now = time.time()
        r = pool.get_url(url)
        elapsed = time.time() - now
        print "Got in %0.3fs: %s" % (elapsed, url)

if __name__ == '__main__':
    print "Running pool_get ..."
    now = time.time()
    pool_get(TO_DOWNLOAD)
    pool_elapsed = time.time() - now

    print "Running urllib_get ..."
    now = time.time()
    urllib_get(TO_DOWNLOAD)
    urllib_elapsed = time.time() - now

    print "Completed pool_get in %0.3fs" % pool_elapsed
    print "Completed urllib_get in %0.3fs" % urllib_elapsed


"""
Example results:

Completed pool_get in 1.163s
Completed urllib_get in 2.318s
"""



#!/usr/bin/env python
# Dummy server used for unit testing

from webob import Request, Response, exc
from cgi import FieldStorage
import time

class TestingApp(object):
    """
    Simple app that performs various operations, useful for testing an HTTP
    library.

    Given any path, it will attempt to convert it will load a corresponding
    local method if it exists. Status code 200 indicates success, 400 indicates
    failure. Each method has its own conditions for success/failure.
    """
    def __call__(self, environ, start_response):
        req = Request(environ)
        target = req.path_info[1:].replace('/','_')
        method = getattr(self, target, self.index)
        resp = method(req)
        return resp(environ, start_response)

    def index(self, request):
        "Render simple message"
        return Response("Dummy server!")

    def specific_method(self, request):
        "Confirm that the request matches the desired method type"
        method = request.params.get('method')
        if request.method != method:
            return Response("Wrong method: %s != %s" % (method, request.method), status='400')
        return Response()

    def upload(self, request):
        "Confirm that the uploaded file conforms to specification"
        param = request.params.get('upload_param', 'myfile')
        filename = request.params.get('upload_filename', '')
        size = int(request.params.get('upload_size', '0'))
        file = request.params.get(param)

        if not isinstance(file, FieldStorage):
            return Response("Not a file: %s" % param, status='400')

        data = file.value
        if int(size) != len(data):
            return Response("Wrong size: %d != %d" % (size, len(data)), status='400')

        if filename != file.filename:
            return Response("Wrong filename: %s != %s" % (filename, file.filename), status='400')

        return Response()

    def redirect(self, request):
        "Perform a redirect to ``target``"
        target = request.params.get('target', '/')
        return exc.HTTPSeeOther(location=target)

    def sleep(self, request):
        "Sleep for a specified amount of ``seconds``"
        seconds = float(request.params.get('seconds', '1'))
        time.sleep(seconds)
        return Response()


def make_server(HOST="localhost", PORT=8081):
    app = TestingApp()
    from wsgiref.simple_server import make_server

    print 'Creating server on http://%s:%s' % (HOST, PORT)
    return make_server(HOST, PORT, app)


if __name__ == '__main__':
    httpd = make_server()
    httpd.serve_forever()



from setuptools import setup, find_packages
import sys, os

version = '0.3'

long_description = open('README.txt').read()

setup(name='urllib3',
      version=version,
      description="HTTP library with thread-safe connection pooling and file post support",
      long_description=long_description,
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='urllib httplib threadsafe filepost http',
      author='Andrey Petrov',
      author_email='andrey.petrov@shazow.net',
      url='http://code.google.com/p/urllib3/',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
